Package: vandalico
Type: Package
Title: Evaluation of Presence-Absence Models
Version: 0.2.2
Date: 2026-04-14
Authors@R: c(person(given = "Alberto", family = "Jiménez-Valverde",
           role = c("aut", "cre"),
           email = "alberto.jimenez.valverde@gmail.com",
           comment = c(ORCID = "0000-0001-9962-2106")))
Maintainer: Alberto Jiménez-Valverde <alberto.jimenez.valverde@gmail.com>
Description: Collection of functions to evaluate presence-absence models. It 
    comprises functions to adjust discrimination statistics for the 
    representativeness effect through case-weighting, along with functions for 
    visualizing the outcomes. Originally outlined in: 
    Jiménez-Valverde (2022) The uniform AUC: dealing with the 
    representativeness effect in presence-absence models. Methods Ecol. Evol, 
    13, 1224-1236.
Acknowledgments: This work was supported by the Ministerio de Ciencia,
        Innovación y Universidades/Agencia Estatal de
        Investigación/FEDER (PID2023-147672NB-I00), and by CSIC
        (2024ICT253).
License: GPL-3
Imports: ROCR (>= 1.0-7), stats, graphics
Depends: R (>= 4.3.0)
RoxygenNote: 7.3.3
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-04-14 08:38:27 UTC; User
Author: Alberto Jiménez-Valverde [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-9962-2106>)
Repository: CRAN
Date/Publication: 2026-04-14 10:20:02 UTC
Built: R 4.4.3; ; 2026-04-14 12:13:42 UTC; unix
