# Generated by roxygen2: do not edit by hand

export(AUCuniform)
export(AUCuniform.2)
export(AUCuniform_trap)
export(CALplot)
export(HSgraph)
importFrom(grDevices,rgb)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,hist)
importFrom(graphics,lines)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(stats,quantile)
importFrom(stats,wilcox.test)
importFrom(utils,head)
importFrom(utils,tail)
